diff mbox series

[v4,7/8] scsi: ufs: core: Toggle Write Booster during clock scaling base on gear speed

Message ID 20250210100212.855127-8-quic_ziqichen@quicinc.com
State New
Headers show
Series Support Multi-frequency scale for UFS | expand

Commit Message

Ziqi Chen Feb. 10, 2025, 10:02 a.m. UTC
From: Can Guo <quic_cang@quicinc.com>

During clock scaling, Write Booster is toggled on or off based on
whether the clock is scaled up or down. However, with OPP V2 powered
multi-level gear scaling, the gear can be scaled amongst multiple gear
speeds, e.g., it may scale down from G5 to G4, or from G4 to G2. To provide
flexibilities, add a new field for clock scaling such that during clock
scaling Write Booster can be enabled or disabled based on gear speeds but
not based on scaling up or down.

Signed-off-by: Can Guo <quic_cang@quicinc.com>
Co-developed-by: Ziqi Chen <quic_ziqichen@quicinc.com>
Signed-off-by: Ziqi Chen <quic_ziqichen@quicinc.com>
Reviewed-by: Bean Huo <beanhuo@micron.com>
Tested-by: Neil Armstrong <neil.armstrong@linaro.org>
---

v1 - > v2:
Initialize the local variables "wb_en" as "false".
v3 -> v4:
1. Add comment for default initialized wb_gear.
2. Remove the unnecessary variable “wb_en" in function
   ufshcd_clock_scaling_unprepare().
---
 drivers/ufs/core/ufshcd.c | 12 ++++++++----
 include/ufs/ufshcd.h      |  3 +++
 2 files changed, 11 insertions(+), 4 deletions(-)

Comments

Peter Wang (王信友) Feb. 11, 2025, 12:59 p.m. UTC | #1
On Mon, 2025-02-10 at 18:02 +0800, Ziqi Chen wrote:
> 
> External email : Please do not click links or open attachments until
> you have verified the sender or the content.
> 
> 
> From: Can Guo <quic_cang@quicinc.com>
> 
> During clock scaling, Write Booster is toggled on or off based on
> whether the clock is scaled up or down. However, with OPP V2 powered
> multi-level gear scaling, the gear can be scaled amongst multiple
> gear
> speeds, e.g., it may scale down from G5 to G4, or from G4 to G2. To
> provide
> flexibilities, add a new field for clock scaling such that during
> clock
> scaling Write Booster can be enabled or disabled based on gear speeds
> but
> not based on scaling up or down.
> 
> Signed-off-by: Can Guo <quic_cang@quicinc.com>
> Co-developed-by: Ziqi Chen <quic_ziqichen@quicinc.com>
> Signed-off-by: Ziqi Chen <quic_ziqichen@quicinc.com>
> Reviewed-by: Bean Huo <beanhuo@micron.com>
> Tested-by: Neil Armstrong <neil.armstrong@linaro.org>
> ---
> 
> v1 - > v2:
> Initialize the local variables "wb_en" as "false".
> v3 -> v4:
> 1. Add comment for default initialized wb_gear.
> 2. Remove the unnecessary variable “wb_en" in function
>    ufshcd_clock_scaling_unprepare().
> ---
>  drivers/ufs/core/ufshcd.c | 12 ++++++++----
>  include/ufs/ufshcd.h      |  3 +++
>  2 files changed, 11 insertions(+), 4 deletions(-)
> 
> diff --git a/drivers/ufs/core/ufshcd.c b/drivers/ufs/core/ufshcd.c
> index bd93119a177d..1276f4a987bd 100644
> --- a/drivers/ufs/core/ufshcd.c
> +++ b/drivers/ufs/core/ufshcd.c
> @@ -1393,13 +1393,13 @@ static int
> ufshcd_clock_scaling_prepare(struct ufs_hba *hba, u64 timeout_us)
>         return ret;
>  }
> 
> -static void ufshcd_clock_scaling_unprepare(struct ufs_hba *hba, int
> err, bool scale_up)
> +static void ufshcd_clock_scaling_unprepare(struct ufs_hba *hba, int
> err)
>  {
>         up_write(&hba->clk_scaling_lock);
> 
> -       /* Enable Write Booster if we have scaled up else disable it
> */
> +       /* Enable Write Booster if current gear requires it else
> disable it */
>         if (ufshcd_enable_wb_if_scaling_up(hba) && !err)
> -               ufshcd_wb_toggle(hba, scale_up);
> +               ufshcd_wb_toggle(hba, hba->pwr_info.gear_rx >= hba-
> >clk_scaling.wb_gear);
> 
>         mutex_unlock(&hba->wb_mutex);
> 
> @@ -1461,7 +1461,7 @@ static int ufshcd_devfreq_scale(struct ufs_hba
> *hba, unsigned long freq,
>         }
> 
>  out_unprepare:
> -       ufshcd_clock_scaling_unprepare(hba, ret, scale_up);
> +       ufshcd_clock_scaling_unprepare(hba, ret);
>         return ret;
>  }
> 
> @@ -1821,6 +1821,10 @@ static void ufshcd_init_clk_scaling(struct
> ufs_hba *hba)
>         if (!hba->clk_scaling.min_gear)
>                 hba->clk_scaling.min_gear = UFS_HS_G1;
> 
> +       if (!hba->clk_scaling.wb_gear)
> +               /* Use intermediate gear speed HS_G3 as the default
> wb_gear */
> +               hba->clk_scaling.wb_gear = UFS_HS_G3;
> +
>         INIT_WORK(&hba->clk_scaling.suspend_work,
>                   ufshcd_clk_scaling_suspend_work);
>         INIT_WORK(&hba->clk_scaling.resume_work,
> diff --git a/include/ufs/ufshcd.h b/include/ufs/ufshcd.h
> index cdb853f5b871..efca700d0520 100644
> --- a/include/ufs/ufshcd.h
> +++ b/include/ufs/ufshcd.h
> @@ -447,6 +447,8 @@ struct ufs_clk_gating {
>   * @resume_work: worker to resume devfreq
>   * @target_freq: frequency requested by devfreq framework
>   * @min_gear: lowest HS gear to scale down to
> + * @wb_gear: enable Write Booster when HS gear scales above or equal
> to it, else
> + *             disable Write Booster
>   * @is_enabled: tracks if scaling is currently enabled or not,
> controlled by
>   *             clkscale_enable sysfs node
>   * @is_allowed: tracks if scaling is currently allowed or not, used
> to block
> @@ -467,6 +469,7 @@ struct ufs_clk_scaling {
>         struct work_struct resume_work;
>         unsigned long target_freq;
>         u32 min_gear;
> +       u32 wb_gear;
>         bool is_enabled;
>         bool is_allowed;
>         bool is_initialized;
> --
> 2.34.1
> 

Reviewed-by: Peter Wang <peter.wang@mediatek.com>
diff --git a/include/ufs/ufshcd.h b/include/ufs/ufshcd.h
index cdb853f5b871..efca700d0520 100644
--- a/include/ufs/ufshcd.h
+++ b/include/ufs/ufshcd.h
@@ -447,6 +447,8 @@  struct ufs_clk_gating {
  * @resume_work: worker to resume devfreq
  * @target_freq: frequency requested by devfreq framework
  * @min_gear: lowest HS gear to scale down to
+ * @wb_gear: enable Write Booster when HS gear scales above or equal to it, else
+ *		disable Write Booster
  * @is_enabled: tracks if scaling is currently enabled or not, controlled by
  *		clkscale_enable sysfs node
  * @is_allowed: tracks if scaling is currently allowed or not, used to block
@@ -467,6 +469,7 @@  struct ufs_clk_scaling {
 	struct work_struct resume_work;
 	unsigned long target_freq;
 	u32 min_gear;
+	u32 wb_gear;
 	bool is_enabled;
 	bool is_allowed;
 	bool is_initialized;
diff mbox series

Patch

diff --git a/drivers/ufs/core/ufshcd.c b/drivers/ufs/core/ufshcd.c
index bd93119a177d..1276f4a987bd 100644
--- a/drivers/ufs/core/ufshcd.c
+++ b/drivers/ufs/core/ufshcd.c
@@ -1393,13 +1393,13 @@  static int ufshcd_clock_scaling_prepare(struct ufs_hba *hba, u64 timeout_us)
 	return ret;
 }
 
-static void ufshcd_clock_scaling_unprepare(struct ufs_hba *hba, int err, bool scale_up)
+static void ufshcd_clock_scaling_unprepare(struct ufs_hba *hba, int err)
 {
 	up_write(&hba->clk_scaling_lock);
 
 	/* Enable Write Booster if we have scaled up else disable it */
 	if (ufshcd_enable_wb_if_scaling_up(hba) && !err)
-		ufshcd_wb_toggle(hba, scale_up);
+		ufshcd_wb_toggle(hba, hba->pwr_info.gear_rx >= hba->clk_scaling.wb_gear);
 
 	mutex_unlock(&hba->wb_mutex);
 
@@ -1461,7 +1461,7 @@  static int ufshcd_devfreq_scale(struct ufs_hba *hba, unsigned long freq,
 	}
 
 out_unprepare:
-	ufshcd_clock_scaling_unprepare(hba, ret, scale_up);
+	ufshcd_clock_scaling_unprepare(hba, ret);
 	return ret;
 }
 
@@ -1821,6 +1821,10 @@  static void ufshcd_init_clk_scaling(struct ufs_hba *hba)
 	if (!hba->clk_scaling.min_gear)
 		hba->clk_scaling.min_gear = UFS_HS_G1;
 
+	if (!hba->clk_scaling.wb_gear)
+		/* Use intermediate gear speed HS_G3 as the default wb_gear */
+		hba->clk_scaling.wb_gear = UFS_HS_G3;
+
 	INIT_WORK(&hba->clk_scaling.suspend_work,
 		  ufshcd_clk_scaling_suspend_work);
 	INIT_WORK(&hba->clk_scaling.resume_work,
@@ -1393,13 +1393,13 @@  static int ufshcd_clock_scaling_prepare(struct ufs_hba *hba, u64 timeout_us)
 	up_write(&hba->clk_scaling_lock);
 
-	/* Enable Write Booster if we have scaled up else disable it */
+	/* Enable Write Booster if current gear requires it else disable it */
 	if (ufshcd_enable_wb_if_scaling_up(hba) && !err)